@@ -38,6 +38,8 @@ CONF = cfg.CONF
 class AgentInspect(common.Common):
     """In-band inspection."""
 
+    default_require_managed_boot = True
+
     def __init__(self):
         super().__init__()
         self.hooks = hooks_base.validate_inspection_hooks()
@@ -74,10 +76,10 @@ class AgentInspect(common.Common):
 
         :param task: a task from TaskManager.
         """
-        if inspect_utils.clear_lookup_addresses(task.node):
-            task.node.save()
-
-        common.clean_up(task, finish=False)
+        error = _("By request, the inspection operation has been aborted")
+        inspect_utils.clear_lookup_addresses(task.node)
+        common.inspection_error_handler(task, error, raise_exc=False,
+                                        clean_up=True)
 
     def continue_inspection(self, task, inventory, plugin_data):
         """Continue in-band hardware inspection.
